﻿using System.Linq;

namespace TabsStudioSelector
{
    public class Selector : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IStyler
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine, EnvDTE80.DTE2 dte)
        {
            this.engine = engine;
            engine.TabCreated += OnTabCreated;
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioSelector=\"clr-namespace:TabsStudioSelector;assembly=Selector\"";
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            SubscribeToTabSelectedChanges(e.Tab.TabItem);
            UpdateAll(e.Tabs);
        }

        private void SubscribeToTabSelectedChanges(System.Windows.Controls.TabItem tabItem)
        {
            if (isTabSelectedDescriptor == null)
                isTabSelectedDescriptor = System.ComponentModel.DependencyPropertyDescriptor.FromName(
                    "IsTabSelected", tabItem.GetType(), tabItem.GetType());
            if (projectNameDescriptor == null)
                projectNameDescriptor = System.ComponentModel.DependencyPropertyDescriptor.FromName(
                    "ProjectName", tabItem.GetType(), tabItem.GetType());

            isTabSelectedDescriptor.AddValueChanged(tabItem, OnTabSelectedChanged);
        }

        private void OnTabSelectedChanged(object sender, System.EventArgs e)
        {
            if ((bool)isTabSelectedDescriptor.GetValue(sender))
            {
                string selectedProject = GetTabProject(sender);

                System.Windows.Controls.TabItem tabItem = sender as System.Windows.Controls.TabItem;
                System.Windows.Controls.Panel panel = tabItem.Parent as System.Windows.Controls.Panel;

                SetTabs(selectedProject, panel);
            }
        }

        private void UpdateAll(TabsStudioExt.Tabs tabs)
        {
            var selectedTab = tabs.TabList.FirstOrDefault(i => i.IsTabSelected);
            string selectedProject = selectedTab == null ? null : GetTabProject(selectedTab.TabItem);
            SetTabs(selectedProject, tabs.TabPanel2);
        }

        private void SetTabs(string selectedProject, System.Windows.Controls.Panel panel)
        {
            foreach (System.Windows.UIElement tab in panel.Children)
                Properties.SetIsSelectedProject(tab, !string.IsNullOrEmpty(selectedProject) && GetTabProject(tab) == selectedProject);
        }

        private string GetTabProject(object tabItem)
        {
            return projectNameDescriptor.GetValue(tabItem) as string;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private System.ComponentModel.DependencyPropertyDescriptor isTabSelectedDescriptor, projectNameDescriptor;
    }
}
